@@ -17,13 +17,13 @@
 %                                April 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -406,10 +406,10 @@ static MagickBooleanType Classify(Image *image,short **extrema,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_Classify)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,SegmentImageTag,progress++,2*
-          image->rows);
+        progress++;
+        proceed=SetImageProgress(image,SegmentImageTag,progress,2*image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
@@ -647,10 +647,10 @@ static MagickBooleanType Classify(Image *image,short **extrema,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_Classify)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,SegmentImageTag,progress++,
-          2*image->rows);
+        progress++;
+        proceed=SetImageProgress(image,SegmentImageTag,progress,2*image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
