@@ -16,13 +16,13 @@
 %                              December 1992                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -122,8 +122,9 @@ MagickPrivate SignatureInfo *AcquireSignatureInfo(void)
   unsigned long
     lsb_first;
 
-  signature_info=(SignatureInfo *) AcquireCriticalMemory(sizeof(*signature_info));
-  (void) ResetMagickMemory(signature_info,0,sizeof(*signature_info));
+  signature_info=(SignatureInfo *) AcquireCriticalMemory(
+    sizeof(*signature_info));
+  (void) memset(signature_info,0,sizeof(*signature_info));
   signature_info->digestsize=SignatureDigestsize;
   signature_info->blocksize=SignatureBlocksize;
   signature_info->digest=AcquireStringInfo(SignatureDigestsize);
@@ -132,6 +133,8 @@ MagickPrivate SignatureInfo *AcquireSignatureInfo(void)
     SignatureBlocksize,sizeof(*signature_info->accumulator));
   if (signature_info->accumulator == (unsigned int *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
+  (void) memset(signature_info->accumulator,0,SignatureBlocksize*
+    sizeof(*signature_info->accumulator));
   lsb_first=1;
   signature_info->lsb_first=(int) (*(char *) &lsb_first) == 1 ? MagickTrue :
     MagickFalse;
@@ -236,14 +239,14 @@ MagickPrivate void FinalizeSignature(SignatureInfo *signature_info)
   datum=GetStringInfoDatum(signature_info->message);
   datum[extent++]=(unsigned char) 0x80;
   if (extent <= (unsigned int) (GetStringInfoLength(signature_info->message)-8))
-    (void) ResetMagickMemory(datum+extent,0,GetStringInfoLength(
+    (void) memset(datum+extent,0,GetStringInfoLength(
       signature_info->message)-8-extent);
   else
     {
-      (void) ResetMagickMemory(datum+extent,0,GetStringInfoLength(
+      (void) memset(datum+extent,0,GetStringInfoLength(
         signature_info->message)-extent);
       TransformSignature(signature_info);
-      (void) ResetMagickMemory(datum,0,GetStringInfoLength(
+      (void) memset(datum,0,GetStringInfoLength(
         signature_info->message)-8);
     }
   datum[56]=(unsigned char) (high_order >> 24);
@@ -584,33 +587,16 @@ MagickExport MagickBooleanType SignatureImage(Image *image,
 %    o signature_info: the address of a structure of type SignatureInfo.
 %
 */
-
-static inline unsigned int Ch(unsigned int x,unsigned int y,unsigned int z)
-{
-  return((x & y) ^ (~x & z));
-}
-
-static inline unsigned int Maj(unsigned int x,unsigned int y,unsigned int z)
-{
-  return((x & y) ^ (x & z) ^ (y & z));
-}
-
-static inline unsigned int Trunc32(unsigned int x)
-{
-  return((unsigned int) (x & 0xffffffffU));
-}
-
-static unsigned int RotateRight(unsigned int x,unsigned int n)
-{
-  return(Trunc32((x >> n) | (x << (32-n))));
-}
-
 static void TransformSignature(SignatureInfo *signature_info)
 {
+#define Ch(x,y,z)  (((x) & (y)) ^ (~(x) & (z)))
+#define Maj(x,y,z)  (((x) & (y)) ^ ((x) & (z)) ^ ((y) & (z)))
+#define RotateRight(x,n)  (Trunc32(((x) >> n) | ((x) << (32-n))))
 #define Sigma0(x)  (RotateRight(x,7) ^ RotateRight(x,18) ^ Trunc32((x) >> 3))
 #define Sigma1(x)  (RotateRight(x,17) ^ RotateRight(x,19) ^ Trunc32((x) >> 10))
 #define Suma0(x)  (RotateRight(x,2) ^ RotateRight(x,13) ^ RotateRight(x,22))
 #define Suma1(x)  (RotateRight(x,6) ^ RotateRight(x,11) ^ RotateRight(x,25))
+#define Trunc32(x)  ((unsigned int) ((x) & 0xffffffffU))
 
   register ssize_t
     i;
@@ -621,7 +607,7 @@ static void TransformSignature(SignatureInfo *signature_info)
   ssize_t
     j;
 
-  static unsigned int
+  static const unsigned int
     K[64] =
     {
       0x428a2f98U, 0x71374491U, 0xb5c0fbcfU, 0xe9b5dba5U, 0x3956c25bU,
@@ -749,7 +735,7 @@ RestoreMSCWarning
   T=0;
   T1=0;
   T2=0;
-  (void) ResetMagickMemory(W,0,sizeof(W));
+  (void) memset(W,0,sizeof(W));
 }
 
 /*
@@ -808,7 +794,7 @@ MagickPrivate void UpdateSignature(SignatureInfo *signature_info,
       i=GetStringInfoLength(signature_info->message)-signature_info->extent;
       if (i > n)
         i=n;
-      (void) CopyMagickMemory(GetStringInfoDatum(signature_info->message)+
+      (void) memcpy(GetStringInfoDatum(signature_info->message)+
         signature_info->extent,p,i);
       n-=i;
       p+=i;
@@ -824,6 +810,6 @@ MagickPrivate void UpdateSignature(SignatureInfo *signature_info,
     n-=GetStringInfoLength(signature_info->message);
     TransformSignature(signature_info);
   }
-  (void) CopyMagickMemory(GetStringInfoDatum(signature_info->message),p,n);
+  (void) memcpy(GetStringInfoDatum(signature_info->message),p,n);
   signature_info->extent=n;
 }
