@@ -17,13 +17,13 @@
 %                               August 2003                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the license.  You may  %
 %  obtain a copy of the license at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  unless required by applicable law or agreed to in writing, software        %
 %  distributed under the license is distributed on an "as is" basis,          %
@@ -59,7 +59,12 @@
 #include "MagickCore/utility-private.h"
 
 /*
-  static declarations.
+  Define declarations.
+*/
+#define CharsPerLine  0x14
+
+/*
+  Static declarations.
 */
 #ifdef __VMS
 #define asciimap AsciiMap
@@ -174,7 +179,7 @@ static StringInfo *AcquireStringInfoContainer()
     *string_info;
 
   string_info=(StringInfo *) AcquireCriticalMemory(sizeof(*string_info));
-  (void) ResetMagickMemory(string_info,0,sizeof(*string_info));
+  (void) memset(string_info,0,sizeof(*string_info));
   string_info->signature=MagickCoreSignature;
   return(string_info);
 }
@@ -239,6 +244,8 @@ MagickExport StringInfo *BlobToStringInfo(const void *blob,const size_t length)
     }
   if (blob != (const void *) NULL)
     (void) memcpy(string_info->datum,blob,length);
+  else
+    (void) memset(string_info->datum,0,length*sizeof(*string_info->datum));
   (void) memset(string_info->datum+length,0,MagickPathExtent*
     sizeof(*string_info->datum));
   return(string_info);
@@ -1192,8 +1199,8 @@ MagickExport ssize_t FormatMagickTime(const time_t time,const size_t length,
     timezone;
 
   assert(timestamp != (char *) NULL);
-  (void) ResetMagickMemory(&local_time,0,sizeof(local_time));
-  (void) ResetMagickMemory(&gm_time,0,sizeof(gm_time));
+  (void) memset(&local_time,0,sizeof(local_time));
+  (void) memset(&gm_time,0,sizeof(gm_time));
 #if defined(MAGICKCORE_HAVE_LOCALTIME_R)
   (void) localtime_r(&time,&local_time);
 #else
@@ -1611,16 +1618,16 @@ MagickExport void PrintStringInfo(FILE *file,const char *id,
     Convert string to a HEX list.
   */
   p=(char *) string_info->datum;
-  for (i=0; i < string_info->length; i+=0x14)
+  for (i=0; i < string_info->length; i+=CharsPerLine)
   {
-    (void) FormatLocaleFile(file,"0x%08lx: ",(unsigned long) (0x14*i));
-    for (j=1; j <= MagickMin(string_info->length-i,0x14); j++)
+    (void) FormatLocaleFile(file,"0x%08lx: ",(unsigned long) (CharsPerLine*i));
+    for (j=1; j <= MagickMin(string_info->length-i,CharsPerLine); j++)
     {
       (void) FormatLocaleFile(file,"%02lx",(unsigned long) (*(p+j)) & 0xff);
       if ((j % 0x04) == 0)
         (void) fputc(' ',file);
     }
-    for ( ; j <= 0x14; j++)
+    for ( ; j <= CharsPerLine; j++)
     {
       (void) fputc(' ',file);
       (void) fputc(' ',file);
@@ -1628,7 +1635,7 @@ MagickExport void PrintStringInfo(FILE *file,const char *id,
         (void) fputc(' ',file);
     }
     (void) fputc(' ',file);
-    for (j=1; j <= MagickMin(string_info->length-i,0x14); j++)
+    for (j=1; j <= MagickMin(string_info->length-i,CharsPerLine); j++)
     {
       if (isprint((int) ((unsigned char) *p)) != 0)
         (void) fputc(*p,file);
@@ -1666,7 +1673,7 @@ MagickExport void ResetStringInfo(StringInfo *string_info)
 {
   assert(string_info != (StringInfo *) NULL);
   assert(string_info->signature == MagickCoreSignature);
-  (void) ResetMagickMemory(string_info->datum,0,string_info->length);
+  (void) memset(string_info->datum,0,string_info->length);
 }
 
 /*
@@ -1751,7 +1758,7 @@ MagickExport void SetStringInfo(StringInfo *string_info,
   assert(source->signature == MagickCoreSignature);
   if (string_info->length == 0)
     return;
-  (void) ResetMagickMemory(string_info->datum,0,string_info->length);
+  (void) memset(string_info->datum,0,string_info->length);
   (void) memcpy(string_info->datum,source->datum,MagickMin(string_info->length,
     source->length));
 }
@@ -2228,6 +2235,8 @@ MagickExport double *StringToArrayOfDoubles(const char *string,ssize_t *count,
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
   *count=0;
+  if (string == (char *) NULL)
+    return((double *) NULL);  /* no value found */
   i=0;
   p=string;
   while (*p != '\0')
@@ -2368,6 +2377,37 @@ MagickExport char *StringToken(const char *delimiters,char **string)
 %
 */
 MagickExport char **StringToList(const char *text)
+{
+  return(StringToStrings(text, (size_t *) NULL));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%  S t r i n g T o S t r i n g s                                              %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  StringToList() converts a text string into a list by segmenting the text
+%  string at each carriage return discovered.  The list is converted to HEX
+%  characters if any control characters are discovered within the text string.
+%
+%  The format of the StringToList method is:
+%
+%      char **StringToList(const char *text,size_t *lines)
+%
+%  A description of each parameter follows:
+%
+%    o text:  Specifies the string to segment into a list.
+%
+%    o count: Return value for the number of items in the list.
+%
+*/
+MagickExport char **StringToStrings(const char *text,size_t *count)
 {
   char
     **textlist;
@@ -2382,7 +2422,11 @@ MagickExport char **StringToList(const char *text)
     lines;
 
   if (text == (char *) NULL)
-    return((char **) NULL);
+    {
+      if (count != (size_t *) NULL)
+        *count=0;
+      return((char **) NULL);
+    }
   for (p=text; *p != '\0'; p++)
     if (((int) ((unsigned char) *p) < 32) &&
         (isspace((int) ((unsigned char) *p)) == 0))
@@ -2409,8 +2453,8 @@ MagickExport char **StringToList(const char *text)
         for (q=p; *q != '\0'; q++)
           if ((*q == '\r') || (*q == '\n'))
             break;
-        textlist[i]=(char *) AcquireQuantumMemory((size_t) (q-p)+
-          MagickPathExtent,sizeof(**textlist));
+        textlist[i]=(char *) AcquireQuantumMemory((size_t) (q-p)+1,
+          sizeof(**textlist));
         if (textlist[i] == (char *) NULL)
           ThrowFatalException(ResourceLimitFatalError,"UnableToConvertText");
         (void) memcpy(textlist[i],p,(size_t) (q-p));
@@ -2434,7 +2478,7 @@ MagickExport char **StringToList(const char *text)
       /*
         Convert string to a HEX list.
       */
-      lines=(size_t) (strlen(text)/0x14)+1;
+      lines=(size_t) (strlen(text)/CharsPerLine)+1;
       textlist=(char **) AcquireQuantumMemory((size_t) lines+1UL,
         sizeof(*textlist));
       if (textlist == (char **) NULL)
@@ -2447,9 +2491,9 @@ MagickExport char **StringToList(const char *text)
         if (textlist[i] == (char *) NULL)
           ThrowFatalException(ResourceLimitFatalError,"UnableToConvertText");
         (void) FormatLocaleString(textlist[i],MagickPathExtent,"0x%08lx: ",
-          (long) (0x14*i));
+          (long) (CharsPerLine*i));
         q=textlist[i]+strlen(textlist[i]);
-        for (j=1; j <= (ssize_t) MagickMin(strlen(p),0x14); j++)
+        for (j=1; j <= (ssize_t) MagickMin(strlen(p),CharsPerLine); j++)
         {
           (void) FormatLocaleString(hex_string,MagickPathExtent,"%02x",*(p+j));
           (void) CopyMagickString(q,hex_string,MagickPathExtent);
@@ -2457,7 +2501,7 @@ MagickExport char **StringToList(const char *text)
           if ((j % 0x04) == 0)
             *q++=' ';
         }
-        for ( ; j <= 0x14; j++)
+        for ( ; j <= CharsPerLine; j++)
         {
           *q++=' ';
           *q++=' ';
@@ -2465,7 +2509,7 @@ MagickExport char **StringToList(const char *text)
             *q++=' ';
         }
         *q++=' ';
-        for (j=1; j <= (ssize_t) MagickMin(strlen(p),0x14); j++)
+        for (j=1; j <= (ssize_t) MagickMin(strlen(p),CharsPerLine); j++)
         {
           if (isprint((int) ((unsigned char) *p)) != 0)
             *q++=(*p);
@@ -2474,8 +2518,14 @@ MagickExport char **StringToList(const char *text)
           p++;
         }
         *q='\0';
+        textlist[i]=(char *) ResizeQuantumMemory(textlist[i],q-textlist[i]+1,
+          sizeof(**textlist));
+        if (textlist[i] == (char *) NULL)
+          ThrowFatalException(ResourceLimitFatalError,"UnableToConvertText");
       }
     }
+  if (count != (size_t *) NULL)
+    *count=lines;
   textlist[i]=(char *) NULL;
   return(textlist);
 }
@@ -2647,9 +2697,9 @@ MagickExport MagickBooleanType SubstituteString(char **string,
       Replace string.
     */
     if (search_extent != replace_extent)
-      (void) CopyMagickMemory(p+replace_extent,p+search_extent,
+      (void) memmove(p+replace_extent,p+search_extent,
         strlen(p+search_extent)+1);
-    (void) CopyMagickMemory(p,replace,replace_extent);
+    (void) memcpy(p,replace,replace_extent);
     p+=replace_extent-1;
   }
   return(status);
