@@ -17,13 +17,13 @@
 %                                 October 1996                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -209,8 +209,7 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
-  threshold_image=CloneImage(image,0,0,MagickTrue,
-    exception);
+  threshold_image=CloneImage(image,0,0,MagickTrue,exception);
   if (threshold_image == (Image *) NULL)
     return((Image *) NULL);
   status=SetImageStorageClass(threshold_image,DirectClass,exception);
@@ -339,9 +338,10 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_AdaptiveThresholdImage)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,AdaptiveThresholdImageTag,progress++,
+        progress++;
+        proceed=SetImageProgress(image,AdaptiveThresholdImageTag,progress,
           image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
@@ -563,7 +563,7 @@ static double OTSUThreshold(const Image *image,const double *histogram,
   return(100.0*threshold/MaxIntensity);
 }
 
-static double TriangleThreshold(const Image *image,const double *histogram,
+static double TriangleThreshold(const double *histogram,
   ExceptionInfo *exception)
 {
   double
@@ -740,7 +740,7 @@ MagickExport MagickBooleanType AutoThresholdImage(Image *image,
     }
     case TriangleThresholdMethod:
     {
-      threshold=TriangleThreshold(image,histogram,exception);
+      threshold=TriangleThreshold(histogram,exception);
       break;
     }
   }
@@ -878,8 +878,9 @@ MagickExport MagickBooleanType BilevelImage(Image *image,const double threshold,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_BilevelImage)
+        #pragma omp atomic
 #endif
+        progress++;
         proceed=SetImageProgress(image,ThresholdImageTag,progress++,
           image->rows);
         if (proceed == MagickFalse)
@@ -1039,9 +1040,10 @@ MagickExport MagickBooleanType BlackThresholdImage(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_BlackThresholdImage)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,ThresholdImageTag,progress++,
+        progress++;
+        proceed=SetImageProgress(image,ThresholdImageTag,progress,
           image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
@@ -1166,9 +1168,10 @@ MagickExport MagickBooleanType ClampImage(Image *image,ExceptionInfo *exception)
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_ClampImage)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,ClampImageTag,progress++,image->rows);
+        progress++;
+        proceed=SetImageProgress(image,ClampImageTag,progress,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
@@ -1815,9 +1818,10 @@ MagickExport MagickBooleanType OrderedDitherImage(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_OrderedDitherImage)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,DitherImageTag,progress++,image->rows);
+        progress++;
+        proceed=SetImageProgress(image,DitherImageTag,progress,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
@@ -1962,9 +1966,11 @@ MagickExport MagickBooleanType PerceptibleImage(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_PerceptibleImage)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,PerceptibleImageTag,progress++,image->rows);
+        progress++;
+        proceed=SetImageProgress(image,PerceptibleImageTag,progress,
+          image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
@@ -2105,9 +2111,10 @@ MagickExport MagickBooleanType RandomThresholdImage(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_RandomThresholdImage)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,ThresholdImageTag,progress++,
+        progress++;
+        proceed=SetImageProgress(image,ThresholdImageTag,progress,
           image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
@@ -2134,27 +2141,27 @@ MagickExport MagickBooleanType RandomThresholdImage(Image *image,
 %  The format of the RangeThresholdImage method is:
 %
 %      MagickBooleanType RangeThresholdImage(Image *image,
-%        const double low_soft,const double high_soft,const double low_hard,
-%        const double high_hard,ExceptionInfo *exception)
+%        const double low_black,const double low_white,const double high_white,
+%        const double high_black,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
 %    o image: the image.
 %
-%    o low_soft: Define the minimum threshold value.
+%    o low_black: Define the minimum threshold value.
 %
-%    o high_soft: Define the maximum threshold value.
+%    o low_white: Define the maximum threshold value.
 %
-%    o low_hard: Define the minimum threshold value.
+%    o high_white: Define the minimum threshold value.
 %
-%    o high_soft: Define the maximum threshold value.
+%    o low_white: Define the maximum threshold value.
 %
 %    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport MagickBooleanType RangeThresholdImage(Image *image,
-  const double low_soft,const double high_soft,const double low_hard,
-  const double high_hard,ExceptionInfo *exception)
+  const double low_black,const double low_white,const double high_white,
+  const double high_black,ExceptionInfo *exception)
 {
 #define ThresholdImageTag  "Threshold/Image"
 
@@ -2221,21 +2228,21 @@ MagickExport MagickBooleanType RangeThresholdImage(Image *image,
           continue;
         if (image->channel_mask != DefaultChannels)
           pixel=(double) q[i];
-        if (pixel < low_soft)
+        if (pixel < low_black)
           q[i]=0;
         else
-          if ((pixel >= low_soft) && (pixel < high_soft))
+          if ((pixel >= low_black) && (pixel < low_white))
             q[i]=ClampToQuantum(QuantumRange*
-              PerceptibleReciprocal(high_soft-low_soft)*(pixel-low_soft));
+              PerceptibleReciprocal(low_white-low_black)*(pixel-low_black));
           else
-            if ((pixel >= high_soft) && (pixel <= low_hard))
+            if ((pixel >= low_white) && (pixel <= high_white))
               q[i]=QuantumRange;
             else
-              if ((pixel > low_hard) && (pixel <= high_hard))
-                q[i]=ClampToQuantum(QuantumRange*
-                  PerceptibleReciprocal(high_hard-low_hard)*(high_hard-pixel));
+              if ((pixel > high_white) && (pixel <= high_black))
+                q[i]=ClampToQuantum(QuantumRange*PerceptibleReciprocal(
+                  high_black-high_white)*(high_black-pixel));
               else
-                if (pixel > high_hard)
+                if (pixel > high_black)
                   q[i]=0;
                 else
                   q[i]=0;
@@ -2250,9 +2257,10 @@ MagickExport MagickBooleanType RangeThresholdImage(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_RangeThresholdImage)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,ThresholdImageTag,progress++,
+        progress++;
+        proceed=SetImageProgress(image,ThresholdImageTag,progress,
           image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
@@ -2411,10 +2419,10 @@ MagickExport MagickBooleanType WhiteThresholdImage(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical (MagickCore_WhiteThresholdImage)
+        #pragma omp atomic
 #endif
-        proceed=SetImageProgress(image,ThresholdImageTag,progress++,
-          image->rows);
+        progress++;
+        proceed=SetImageProgress(image,ThresholdImageTag,progress,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
