@@ -280,7 +280,7 @@ static void InsertComplexFloatRow(Image *image,float *p,int y,double MinVal,
     if (*p < 0)
       {
         f=(*p/MaxVal)*(Quantum) (QuantumRange-GetPixelBlue(image,q));
-        if ((f > 0) && ((f+GetPixelBlue(image,q)) < QuantumRange))
+        if ((f+GetPixelBlue(image,q)) < QuantumRange)
           SetPixelBlue(image,GetPixelBlue(image,q)+ClampToQuantum(f),q);
         else
           SetPixelBlue(image,QuantumRange,q);
@@ -1117,7 +1117,9 @@ MATLAB_KO:
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),
         "MATLAB_HDR.CellType: %.20g",(double) CellType);
 
-    (void) ReadBlob(image2, 4, (unsigned char *) &size);     /* data size */
+    /* data size */
+    if (ReadBlob(image2, 4, (unsigned char *) &size) != 4)
+      goto MATLAB_KO;
 
     NEXT_FRAME:
     switch (CellType)
