@@ -51,6 +51,7 @@
 #include "MagickCore/magick.h"
 #include "MagickCore/memory_.h"
 #include "MagickCore/quantum-private.h"
+#include "MagickCore/resource_.h"
 #include "MagickCore/static.h"
 #include "MagickCore/string_.h"
 #include "MagickCore/module.h"
@@ -252,7 +253,8 @@ static Image *ReadTTFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   */
   y=20;
   draw_info=CloneDrawInfo(image_info,(DrawInfo *) NULL);
-  draw_info->font=AcquireString(image->filename);
+  draw_info->font=AcquireString("");
+  (void) ImageToFile(image,draw_info->font,exception);
   ConcatenateString(&draw_info->primitive,"push graphic-context\n");
   (void) FormatLocaleString(buffer,MagickPathExtent,
     " viewbox 0 0 %.20g %.20g\n",(double) image->columns,(double) image->rows);
@@ -288,6 +290,7 @@ static Image *ReadTTFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Relinquish resources.
   */
+  (void) RelinquishUniqueFileResource(draw_info->font);  
   draw_info=DestroyDrawInfo(draw_info);
   (void) CloseBlob(image);
   return(GetFirstImageInList(image));
